/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.authentication.loginmodule;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Map.Entry;

import org.apache.log4j.Logger;
import org.n52.security.authentication.Credential;
import org.n52.security.authentication.SAMLCredential;
import org.n52.security.authentication.SAMLTicket;
import org.opensaml.SAMLException;

/**
 * Caches Credentials identified by an ID string. Cache entries are cleaned up
 * if they were added a number of seconds in the past, defined by
 * {@link #setTokenTimeout(long)}. The cache is allowed to grow up to a number
 * of <code>n</code> entries defined by {@link #setMaxTokens(int)}. Right before
 * the <code>n+1</code> entry is added the cash is completely cleaned.
 * 
 * @author drewnak
 * 
 */
/**
 * @author jdr
 *
 */
public class CredentialCache {

    private static final Logger LOG = Logger.getLogger(CredentialCache.class);

    private static final boolean LOG_IS_DEBUG = LOG.isDebugEnabled();

    private Map m_tokensByCredentials;


    /**
     * Token timeout in [sec].
     */
    private long m_tokenTimeoutSec;

    /**
     * Token timeout in [msec].
     */
    private long m_tokenTimeoutMillis;

    private int m_maxTokens;

    private Timer m_cleanupTimer;

    /**
     * 
     */
    public CredentialCache() {
        m_tokensByCredentials = new HashMap();
        m_cleanupTimer = new Timer("CredentialCacheCleaner", true);
        setTokenTimeout(30);
    }

    /**
     * Initializes cache. Starts a new {@link CredentialCacheCleaner}.
     */
    public void init() {
        m_cleanupTimer.schedule(new CredentialCacheCleaner(), 0, m_tokenTimeoutMillis);
        LOG.info("CredentialCache initialized " + toString());
    }
    
    public void destroy() {
        m_cleanupTimer.cancel();
        LOG.info("CredentialCache destroyed, cleanup task cancelled.");
    }

    /**
     * Set token timeout in [sec].
     * 
     * @param tokenTimeout
     *            the tokenTimeout to set
     */
    public synchronized void setTokenTimeout(final long tokenTimeout) {
        m_tokenTimeoutSec = tokenTimeout;
        m_tokenTimeoutMillis = tokenTimeout * 1000;
    }

    /**
     * @return the tokenTimeout
     */
    public long getTokenTimeout() {
        return m_tokenTimeoutSec;
    }

    /**
     * @return the maxTokens
     */
    public int getMaxTokens() {
        return m_maxTokens;
    }

    /**
     * @param maxTokens
     *            the maxTokens to set
     */
    public synchronized void setMaxTokens(final int maxTokens) {
        m_maxTokens = maxTokens;
    }
    
    public synchronized void put(final CredentialCacheEntry entry) {
        if (m_tokensByCredentials.size() >= m_maxTokens) {
            m_tokensByCredentials.clear();
        }
        entry.setEntryTime(System.currentTimeMillis());
        m_tokensByCredentials.put(entry.getId(), entry);
    }

    /**
     * @param cacheId
     * @return
     */
    public synchronized CredentialCacheEntry get(final String cacheId) {
        CredentialCacheEntry entry = (CredentialCacheEntry) m_tokensByCredentials.get(cacheId);

        if (entry == null) {
            return null;
        }

        // Check if entry is still valid.
        long tokenLimit = System.currentTimeMillis() - m_tokenTimeoutMillis;
        if (entry.getEntryTime() < tokenLimit || entry.isInternallyExpired()) {
            remove(entry.getId());
            return null;
        }

        return entry;
    }

    public synchronized CredentialCacheEntry remove(final String cacheId) {
        return (CredentialCacheEntry) m_tokensByCredentials.remove(cacheId);
    }

    /**
     * This {@link TimerTask} extension cleans the CredentialCache when run. It iterates through 
     * the list of cached entries and deletes every cache entry that is either expired by means of
     * credential cache expiration or if {@link CredentialCacheEntry#isInternallyExpired()} is <code>true</code>. 
     *
     */
    private class CredentialCacheCleaner extends TimerTask {

        public void run() {
            //require monitor of CredentialCache instance
            synchronized (CredentialCache.this) {
                if (m_tokensByCredentials.size() == 0) {
                    return;
                }
                long tokenLimit = System.currentTimeMillis() - m_tokenTimeoutMillis;
                for (Iterator iterator = m_tokensByCredentials.entrySet().iterator(); iterator.hasNext();) {

                    Entry entry = (Entry) iterator.next();
                    CredentialCacheEntry cacheEntry = (CredentialCacheEntry) entry.getValue();
                    if (cacheEntry.getEntryTime() < tokenLimit || cacheEntry.isInternallyExpired()) {
                        String cacheId = cacheEntry.getId();
                        iterator.remove();
                        if (LOG_IS_DEBUG) {
                            LOG.debug("Removed cached credential at " + cacheId + " due to expiration");
                        }

                    }
                }
            }

        }

    }

    /**
     * An implementation of {@link CredentialCacheEntry} that contains
     * {@link SAMLCredential} instances.
     * 
     * @author drewnak
     * 
     */
    public static class SAMLCredentialCacheEntry extends CredentialCacheEntry {

        private final String m_id;

        private final SAMLCredential m_token;
        
        /**
         * Creates a cache entry for a SAML credential that is linked to a
         * string denoting, e.g. another credential. credential.
         * 
         * @param idCred
         *            String that is used to identify a SAML credential.
         * @param cachedEntry
         *            the SAML credential that is cached.
         */
        public SAMLCredentialCacheEntry(final String id, final SAMLCredential cachedEntry) {
            this.m_id = id;
            m_token = cachedEntry;
        }

        /*
         * (non-Javadoc)
         * 
         * @see
         * org.n52.security.authentication.loginmodule.CredentialCacheEntry#
         * getId()
         */
        public String getId() {
            return m_id;
        }

        /* (non-Javadoc)
         * @see org.n52.security.authentication.loginmodule.CredentialCacheEntry#expiresInternally()
         */
        public boolean internallyExpires() {
            return true;
        }

        /* (non-Javadoc)
         * @see org.n52.security.authentication.loginmodule.CredentialCacheEntry#isInternallyExpired()
         */
        public boolean isInternallyExpired() {
            return m_token.getSAMLTicket().isExpired();
        }

        /* (non-Javadoc)
         * @see org.n52.security.authentication.loginmodule.CredentialCacheEntry#getCredentialCopy()
         */
        public Object getCredentialCopy() {

            SAMLTicket ticket;
            try {
                ticket = new SAMLTicket(m_token.getSAMLTicket().asString());
            } catch (SAMLException e) {
                LOG.warn("could not create a copy of SAMLTicket", e);
                return null;
            }
            return new SAMLCredential(ticket);
        }

        /* (non-Javadoc)
         * @see org.n52.security.authentication.loginmodule.CredentialCacheEntry#getCredential()
         */
        public Credential getCredential() {
            return m_token;
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "[maxTokens=" + m_maxTokens + ", tokenTimeout=" + m_tokenTimeoutSec + ", current size="
                + m_tokensByCredentials.size() + "]";
    }

}
