/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.authentication.loginmodule;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import org.apache.log4j.Logger;
import org.n52.security.authentication.UsernamePasswordCredential;
import org.n52.security.authentication.callbacks.CredentialCallback;
import org.n52.security.authentication.principals.LoginNamePrincipal;
import org.n52.security.authentication.principals.RolePrincipal;
import org.n52.security.authentication.principals.UsernameIDPrincipal;
import org.n52.security.common.util.StringUtils;

/**
 * Pretty simple login module to log in users by means of a password checked against a database connected via JDBC.
 * <p>
 * <b>!!!Caution!!! This login module is plain and simple. It is not recommended to be used in productive
 * envireonments</b>
 * </p>
 * Initialization options are:
 * <ul>
 * <li> <code>db.driver.class</code>: JDBC driver class used for database connection
 * <li> <code>db.url</code>: JDBC connection URL
 * <li> <code>db.user</code>: Database user used for connection
 * <li> <code>db.password</code>: Database user's password
 * </ul>
 * 
 * The database schema must be named <code>Users</code> and contain the following string rows:
 * <ul>
 * <li> <code>Name</code>: User's name, mapped to role attribute used by policy decision system
 * <li> <code>EMail</code>: User's mail address, might be empty
 * <li> <code>Password</code>: User's password in plain text
 * <li> <code>Username</code>: Username used to log in.
 * </ul>
 * 
 * 
 * @author jdr
 * 
 */
public class DataBaseLoginModule implements LoginModule {

    private static Connection sDBConnection = null;

    private static Logger sLogger = Logger.getLogger(DataBaseLoginModule.class);

    private Subject m_subject;

    private CallbackHandler m_callbackHandler;

    private Map m_sharedState;

    private Map m_options;

    private boolean m_debug = false;

    private Set m_principals;

    /*
     * (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#abort()
     */
    public boolean abort() throws LoginException {
        m_subject = null;
        m_callbackHandler = null;
        m_sharedState = null;
        m_options = null;
        m_debug = false;
        m_principals.clear();

        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#commit()
     */
    public boolean commit() throws LoginException {
        m_subject.getPrincipals().addAll(m_principals);
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#login()
     */
    public boolean login() throws LoginException {

        Statement stmt = null;
        ResultSet rs = null;

        boolean isBase64Encoded = Boolean.valueOf((String) m_options.get("credential.isBase64Encoded")).booleanValue();

        try {

            // get the initialization params

            if (sDBConnection == null) {
                String dbDriverClass = (String) m_options.get("db.driver.class");
                String dbURL = (String) m_options.get("db.url");
                String dbUser = (String) m_options.get("db.user");
                String dbPassword = (String) m_options.get("db.password");
                sLogger.info("Creating database connection " + dbURL + ", " + dbUser + ", " + dbPassword);
                Class.forName(dbDriverClass);
                sDBConnection = DriverManager.getConnection(dbURL, dbUser, dbPassword);
            }

            // get the credentials
            Set crendentialSet = m_subject.getPublicCredentials();
            Iterator it = crendentialSet.iterator();
            UsernamePasswordCredential credential = null;
            while (it.hasNext()) {
                Object publicCred = it.next();
                if (publicCred instanceof UsernamePasswordCredential) {
                    credential = (UsernamePasswordCredential) it.next();
                    break;
                }
            }
            if (credential == null) {
                // use callbackhandler
                CredentialCallback credentialCallback = new CredentialCallback(UsernamePasswordCredential.class);
                m_callbackHandler.handle(new Callback[] { credentialCallback });
                credential = (UsernamePasswordCredential) credentialCallback.getCredential();
                if (credential == null) {
                    throw new LoginException("no required credentials found");
                }
            }

            // check if credentials are given properly

            String user = credential.getUsername();
            String pass = new String(credential.getPassword());

            if (isBase64Encoded) {
                pass = StringUtils.decodeBase64(pass);
                user = StringUtils.decodeBase64(user);

            }
            sLogger.debug("Trying to authenticate " + user);

            stmt = sDBConnection.createStatement();
            String statement = "SELECT Name, EMail, Password FROM Users WHERE Username = '" + user + "';";
            rs = stmt.executeQuery(statement);

            boolean tUserExists = rs.next();
            if (!tUserExists) {
                throw new LoginException("Unknown login name");
            }

            if (!rs.getString("Password").equals(pass)) {
                throw new LoginException("Wrong password");
            }

            m_principals.add(new LoginNamePrincipal(user));
            m_principals.add(new UsernameIDPrincipal(user));
            m_principals.add(new RolePrincipal(rs.getString("Name")));
            return true;

        } catch (ClassNotFoundException cnfe) {
            sLogger.error("Database driver class not found", cnfe);
            throw new LoginException("Internal service error. Please contact the service administrator.");
        } catch (SQLException se) {
            sLogger.error("Server-side database _access_ error: " + se.getMessage(), se);
            throw new LoginException("Internal service error. Please contact the service administrator.");

        } catch (UnsupportedCallbackException e) {
            throw new LoginException("CallbackHandler supports no CredentialCallbacks.");
        } catch (IOException e) {
            throw new LoginException("Internal service error. Please contact the service administrator.");
        } finally {
            try {
                rs.close();
                stmt.close();
            } catch (Exception e) {
                sLogger.error("Error during login process", e);
                throw new LoginException("Internal service error. Please contact the service administrator.");

            }
        }
    }

    /*
     * (non-Javadoc) /* (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#logout()
     */
    public boolean logout() throws LoginException {
        Iterator it = m_subject.getPrincipals().iterator();

        while (it.hasNext()) {
            m_subject.getPrincipals().remove(it.next());
        }
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.security.auth.spi.LoginModule#initialize(javax.security.auth.Subject,
     * javax.security.auth.callback.CallbackHandler, java.util.Map, java.util.Map)
     */
    public void initialize(Subject subject, CallbackHandler callbackHandler, Map sharedState, Map options) {
        this.m_subject = subject;
        this.m_callbackHandler = callbackHandler;
        this.m_sharedState = sharedState;
        this.m_options = options;
        m_debug = "true".equalsIgnoreCase((String) options.get("debug"));
        this.m_principals = new HashSet(8);
    }
}
