/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Author: Jan Drewnak
 * Modified: $Date $
 ******************************************************************************/

package org.n52.security.authentication.loginmodule;

import java.security.Principal;
import java.util.Iterator;
import java.util.Set;
import javax.security.auth.callback.Callback;
import javax.security.auth.login.CredentialExpiredException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.n52.security.authentication.SAMLCredential;
import org.n52.security.authentication.SAMLTicket;
import org.n52.security.authentication.callbacks.CredentialCallback;
import org.n52.security.authentication.principals.AttributePrincipal;
import org.n52.security.authentication.principals.RolePrincipal;
import org.n52.security.common.crypto.KeyPair;
import org.opensaml.SAMLException;

/**
 * Authenticates users by checking a SAMLResponse (or better an instance of {@link SAMLTicket}).
 * Adds a list of {@link RolePrincipal} to the subject.
 * 
 * @author Marko Reiprecht
 *         Created on 25.07.2008
 * @version $Revision: 1755 $
 */
public class SAMLTicketLoginModule extends AbstractLoginModule {

    // ------------------------------ FIELDS ------------------------------
    /**
     * validation key pair option identifier.
     */
    public static final String OPTION_VALIDATION_KEYPAIR = "validationKeyPair";
    /**
     * no validation option identifier.
     */
    public static final String OPTION_NO_VALIDATION = "noValidation";
    /**
     * expired timeset offset option identifier.
     */
    public static final String OPTION_EXPIRED_TIME_OFFSET = "expiredTimeOffset";
    /**
     * SAMLAttribute role name option identifier.
     */
    public static final String OPTION_SAML_ROLE_ATTRIBUTE_NAME = "SAMLRoleAttributeName";

    /**
     * The Logger.
     */
    private static final Log LOG = LogFactory.getLog(SAMLTicketLoginModule.class);

    private static final long serialVersionUID = 2631946285948582729L;

    private KeyPair m_validationKeyPair;
    private SAMLCredential m_samlCredential;
    private long m_expiredTimeOffset;
    private boolean m_ticketFromPublicCredentials;
    private String m_samlRoleAttributeName;

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * @see AbstractLoginModule#getDescription()
     */
    protected String getDescription() {
        return new StringBuffer(150)
                .append("The authentication is performed through the '")
                .append(SAMLTicketLoginModule.class.getName())
                .append("' login module. It requires a SAML Response.").toString();
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * @see AbstractLoginModule#clearAuthenticationState()
     */
    protected void clearAuthenticationState() throws LoginException {
        m_ticketFromPublicCredentials = false;
        m_samlCredential = null;
    }

    /**
     * Initializes the login module.
     * It looks for the option "validationKeyPair" which must be an instance of KeyPair.
     * If the option "noValidation" is set to true the key pair is ignored and
     * no validation of the saml assertion is done.
     * <p/>
     * The Option SAMLRoleAttributeName is used to convert attribute principals to role principals.
     *
     * @see AbstractLoginModule#initialize()
     */
    protected void initialize() {
        Options options = getOptions();
        if (!options.is(OPTION_NO_VALIDATION, false)) {
            m_validationKeyPair = (KeyPair) options.getAs("validationKeyPair", KeyPair.class);
            if (!m_validationKeyPair.isCertificateSet() && !m_validationKeyPair.isCertificateChainSet()) {
                throw new IllegalArgumentException("The property <validationKeyPair> contains no valid certificate");
            }
        }
        m_samlRoleAttributeName = options.getAsString(OPTION_SAML_ROLE_ATTRIBUTE_NAME, "");
        // default 5sec time difference is allowed.
        m_expiredTimeOffset = options.getAsLong(OPTION_EXPIRED_TIME_OFFSET, 5000);
    }

    /**
     * @see AbstractLoginModule#performLogin()
     */
    public boolean performLogin() throws LoginException {
        // shared state not used yet.
        // SharedState sharedState = getSharedState();

        // use callbackhandler
        CredentialCallback credentialCallback = new CredentialCallback(SAMLCredential.class);
        handleCallbacks(new Callback[]{credentialCallback});
        SAMLCredential credential = (SAMLCredential) credentialCallback.getCredential();
        if (credential == null) {
            // use allready available public credentials of the given subject.
            Set crendentialSet = getSubject().getPublicCredentials(SAMLCredential.class);
            Iterator it = crendentialSet.iterator();
            if (it.hasNext()) {
                // if there is one use it
                credential = (SAMLCredential) it.next();
                m_ticketFromPublicCredentials = true;
            }
        }
        if (credential == null) {
            // seems that no credentials for this module are present, so this module goes to do nothing
            // may be other modules are able to authenticate the user.
            if (LOG.isInfoEnabled()) {
                LOG.info("No credentials for module <" + getClass().getName() + "> available, skip login.");
            }
            return false;
        }
        SAMLTicket ticket = credential.getSAMLTicket();
        try {
            // now check the signature
            if (m_validationKeyPair != null) {
                ticket.verify(m_validationKeyPair.getCertificate());
            }
        } catch (SAMLException se) {
            throw new FailedLoginException(
                    "Invalid SAML Credential. Reason: " + se);
        }
        // check the expiration
        if (ticket.isExpired(m_expiredTimeOffset)) {
            throw new CredentialExpiredException("SAML Credential expired");
        }
        // store credential for commit phase
        m_samlCredential = credential;
        return true;
    }

    /**
     * @see AbstractLoginModule#prepareCommitState()
     */
    protected void prepareCommitState() throws LoginException {
        if (!m_ticketFromPublicCredentials) {
            addPublicCredential(m_samlCredential);
        }
        // todo: the refactor the SAML-Ticket class !!! (e.g. roles are problematic)
        Set lPrincipals = m_samlCredential.getSAMLTicket().asSubject().getPrincipals();
        for (Iterator iterator = lPrincipals.iterator(); iterator.hasNext();) {
            Principal principal = (Principal) iterator.next();
            if (principal instanceof AttributePrincipal && m_samlRoleAttributeName.equals((principal).getName())) {
                addPrincipal(new RolePrincipal(((AttributePrincipal) principal).getValue()));
            } else {
                addPrincipal(principal);
            }
        }
    }
}