/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 05.08.2008
 * Modified: $Date $
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.attributes;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * Class Attributes are collections of attributes.
 *
 * @author Marko Reiprecht
 *         Created on 05.08.2008
 * @version $Revision: $
 */
public class Attributes implements Serializable {

    // ------------------------------ FIELDS ------------------------------

    private static final long serialVersionUID = -6724866345688113007L;

    /**
     * Stores attributes.
     */
    protected Map m_attributes;

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Constructor creates a new instance.
     */
    public Attributes() {
        m_attributes = new HashMap();
    }

    /**
     * Constructor creates a new instance.
     *
     * @param attributes the attributes.
     */
    public Attributes(final Collection attributes) {
        if (attributes == null) {
            throw new IllegalArgumentException("<attributes> must not null");
        }
        m_attributes = AttributeUtil.fillAttributeMap(attributes);
    }

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Gets the attributes.
     * The collection is unmodifiable.
     *
     * @return returns the attributes.
     */
    public Collection getAttributes() {
        return Collections.unmodifiableCollection(m_attributes.values());
    }

    /**
     * Gets the attribute names.
     *
     * @return returns the attributenames.
     */
    public Set getAttributeNames() {
        return Collections.unmodifiableSet(m_attributes.keySet());
    }

    /**
     * Method says if environment is empty.
     *
     * @return boolean.
     */
    public boolean isEmpty() {
        return m_attributes.isEmpty();
    }

    // ------------------------ CANONICAL METHODS ------------------------

    /**
     * Method equals.
     *
     * @param o the o.
     * @return boolean.
     */
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof Attributes)) {
            return false;
        }

        final Attributes that = (Attributes) o;

        if (!m_attributes.equals(that.m_attributes)) {
            return false;
        }

        return true;
    }

    /**
     * Method hashCode.
     *
     * @return int.
     */
    public int hashCode() {
        return m_attributes.hashCode();
    }

    /**
     * Method toString.
     *
     * @return String.
     */
    public String toString() {
        final StringBuffer sb = new StringBuffer();
        sb.append("{").append(getAttributes()).append('}');
        return sb.toString();
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Gets an Attribute for a given key.
     *
     * @param key the key of the attribute.
     * @return the found attribute or null if nothing is found.
     */
    public Attribute getAttribute(final String key) {
        return (Attribute) m_attributes.get(key);
    }

    /**
     * Gets the value of an Attribute with StringAttributeValue.
     *
     * @param key the key of the attribute.
     * @return the found attribute value or null if not set.
     */
    public String getStringAttributeValue(final String key) {
        if (containsAttribute(key)) {
            return ((StringAttributeValue) getAttribute(key).getValue()).getValue();
        }
        return null;
    }

    /**
     * Returns true if an attribute with the given key is present.
     *
     * @param key a key, asserted to be not null.
     * @return true if attribute with key is present.
     */
    public boolean containsAttribute(final String key) {
        return m_attributes.containsKey(key);
    }

    /**
     * Method puts an attribute in the environment.
     *
     * @param attribute the attribute.
     * @return Environment.
     */
    public Attributes putAttribute(final Attribute attribute) {
        if (attribute == null) {
            throw new IllegalArgumentException("<attribute> must not null");
        }
        m_attributes.put(attribute.getKey(), attribute);
        return this;
    }

    /**
     * Method puts a simple StringAttribute in the environment.
     *
     * @param key   the key.
     * @param value the value.
     * @return Environment.
     */
    public Attributes putAttribute(final String key, final String value) {
        if (key == null) {
            throw new IllegalArgumentException("<key> must not null");
        }
        if (value == null) {
            throw new IllegalArgumentException("<value> must not null");
        }
        return putAttribute(new Attribute(key, new StringAttributeValue(value)));
    }

    /**
     * Method puts a collection of attributes in the environment.
     *
     * @param attributes the attributes.
     * @return Environment.
     */
    public Attributes putAttributes(final Collection attributes) {
        if (attributes == null) {
            throw new IllegalArgumentException("<attributes> must not null");
        }
        m_attributes = AttributeUtil.fillAttributeMap(m_attributes, attributes);
        return this;
    }

    /**
     * Method removes the attribute with the given key.
     *
     * @param key the key.
     * @return Environment.
     */
    public Attributes removeAttribute(final String key) {
        if (key == null) {
            throw new IllegalArgumentException("<key> must not null");
        }
        m_attributes.remove(key);
        return this;
    }

    /**
     * Method removes all attributes in the collection.
     *
     * @param attributes the attributes.
     * @return Environment.
     */
    public Attributes removeAttributes(final Collection attributes) {
        if (attributes == null) {
            throw new IllegalArgumentException("<attributes> must not null");
        }
        for (Iterator iterator = attributes.iterator(); iterator.hasNext();) {
            Attribute attribute = (Attribute) iterator.next();
            removeAttribute(attribute);
        }
        return this;
    }

    /**
     * Method removes the attribute from the environment.
     *
     * @param attribute the attribute.
     * @return Environment.
     */
    public Attributes removeAttribute(final Attribute attribute) {
        if (attribute == null) {
            throw new IllegalArgumentException("<attribute> must not null");
        }
        m_attributes.remove(attribute.getKey());
        return this;
    }

    /**
     * Method gets the size.
     *
     * @return int.
     */
    public int size() {
        return m_attributes.size();
    }
}
