/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 24.03.2008
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.xml;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;

/**
 * Class DOMParser encapsulates the common usage of the DocumentBuilder class within a this helpfull DOMParser class.
 * <p/>
 * At first it is designed for performance, such as you can discover a pooling DomParser, what is recommended.
 * Only in very few situations you need a non pooling DOMParser.
 * <p/>
 * To simple use simple use the DOMParser.createNew() method, than you use the best performance.
 * <p/>
 * At next the DOMParser class uses a ClasspathEntityResolver, which ensures that the wrapped
 * DocumentBuilder can always parse an XMLFile, and entities are only discovered within the classpath.
 * <p/>
 * The DOMParser class uses the factory method pattern, there with you can create DOMParsers by using the
 * static methodds {@link #createPoolingParser()} and {@link #createNonPoolingParser()}.
 * <p/>
 * For fast creation of empty xml documents, or for in memory creation of DOM documents use the
 * {@link #createEmptyDocument()} method.
 *
 * @author Marko Reiprecht
 *         Created on 24.03.2008
 * @version $Revision: $
 */
public abstract class DOMParser {

    // ------------------------------ FIELDS ------------------------------

    private static EntityResolver defaultEntityResolver = new ClasspathEntityResolver(true);
    private static DocumentBuilderPool documentBuilderPool = new DefaultDocumentBuilderPool();

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Method gets the defaultEntityResolver.
     * <p/>
     * By default this method creates a {@link ClasspathEntityResolver} in quite mode,
     * which means the parsing will never fail because of non existing entities (e.g. dtd's).
     *
     * @return EntityResolver.
     */
    public static EntityResolver getDefaultEntityResolver() {
        return defaultEntityResolver;
    }

    /**
     * Method sets the defaultEntityResolver.
     * <p/>
     * the default entity resolver is set on each new created DocumentBuilder.
     *
     * @param defaultresolver the defaultEntityResolver.
     */
    public static void setDefaultEntityResolver(final EntityResolver defaultresolver) {
        defaultEntityResolver = defaultresolver;
    }

    /**
     * Method gets the documentBuilderPool used by pooled parsers.
     *
     * @return DocumentBuilderPool.
     */
    private static DocumentBuilderPool getDocumentBuilderPool() {
        return documentBuilderPool;
    }

    /**
     * Method sets the documentBuilderPool.
     *
     * @param builderPool the documentBuilderPool.
     */
    public static void setDocumentBuilderPool(final DocumentBuilderPool builderPool) {
        documentBuilderPool = builderPool;
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Method creates a new DocumentBuilder with the given attributes.
     * <p/>
     *
     * @param parseroptions options for the new parser.
     * @return DocumentBuilder.
     */
    protected static DocumentBuilder createDocumentBuilder(final DOMParserOptions parseroptions) {
        DocumentBuilderFactory fac = DocumentBuilderFactory.newInstance();
        DOMParserOptions options = parseroptions;
        if (options == null) {
            options = DOMParserOptions.getDefault();
        }

        fac.setValidating(options.isValidating());
        fac.setNamespaceAware(options.isNamespaceaware());
        fac.setIgnoringComments(options.isIgnoreComments());
        try {
            return fac.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * Method creates an empty Document.
     *
     * @return Document.
     */
    public static Document createEmptyDocument() {
        return createEmptyDocument(null);
    }

    /**
     * Method creates an empty Document for the given options.
     *
     * @param options the options.
     * @return Document.
     */
    public static Document createEmptyDocument(final DOMParserOptions options) {
        DOMParserOptions opts = options != null ? options : DOMParserOptions.getDefault();
        DocumentBuilderPool pool = getDocumentBuilderPool();
        DocumentBuilder builder = pool.acquire(opts);
        try {
            return builder.newDocument();
        } finally {
            pool.release(builder, opts);
        }
    }

    /**
     * Method creates a new DOMParser.
     * This method shall be used by dependend classes,
     * it will delegate to createPoolingParser() methods, so it uses the fastest approach.
     *
     * @return DOMParser.
     */
    public static DOMParser createNew() {
        return createPoolingParser();
    }

    /**
     * Method creates a new DOMParser.
     * This method shall be used by dependend classes,
     * it will delegate to createPoolingParser() methods, so it uses the fastest aproach.
     *
     * @param options the options.
     * @return DOMParser.
     */
    public static DOMParser createNew(final DOMParserOptions options) {
        return createPoolingParser(options);
    }

    /**
     * Method creates a non pooling Parser, with default options.
     *
     * @return DOMParser.
     */
    public static DOMParser createNonPoolingParser() {
        return createNonPoolingParser(null);
    }

    /**
     * Method creates a non pooling Parser, with the given options.
     *
     * @param options the options.
     * @return DOMParser.
     */
    public static DOMParser createNonPoolingParser(final DOMParserOptions options) {
        return new NonPoolingDOMParser(options != null ? options : DOMParserOptions.getDefault());
    }

    /**
     * Method creates a pooling Parser with default options and using the global DocumentBuilder pool.
     *
     * @return DOMParser.
     */
    public static DOMParser createPoolingParser() {
        return createPoolingParser(null);
    }

    /**
     * Method creates a pooling Parser with the given options, using the global DocumentBuilder pool.
     *
     * @param options the options.
     * @return DOMParser.
     */
    public static DOMParser createPoolingParser(final DOMParserOptions options) {
        return new PoolingDOMParser(options != null ? options : DOMParserOptions.getDefault(),
                                    getDocumentBuilderPool());
    }

    /**
     * Method creates a pooling Parser with the given options, which uses the given document builder pool.
     *
     * @param options the options.
     * @param pool    the document builder pool.
     * @return DOMParser.
     */
    public static DOMParser createPoolingParser(final DOMParserOptions options,
                                                final DocumentBuilderPool pool) {
        return new PoolingDOMParser(options != null ? options : DOMParserOptions.getDefault(),
                                    pool != null ? pool : getDocumentBuilderPool());
    }

    /**
     * Method parses the inputsource and informs the error handler about errors.
     *
     * @param inputSource  the inputSource.
     * @param errorhandler the errorhandler.
     * @return Document.
     * @throws DOMParserException if an SAXException or IOException occurs.
     */
    public Document parse(final InputSource inputSource, final ErrorHandler errorhandler)
            throws DOMParserException {
        return parse(inputSource, errorhandler, getDefaultEntityResolver());
    }

    /**
     * Method parses the inputsource using the errorhandler and entityresolver.
     *
     * @param source         the source.
     * @param errorhandler   the errorhandler.
     * @param entityResolver the entityResolver.
     * @return Document.
     * @throws DOMParserException if an SAXException or IOException occurs.
     */
    public abstract Document parse(InputSource source, ErrorHandler errorhandler, EntityResolver entityResolver)
            throws DOMParserException;

    /**
     * Method parses the inputsource.
     *
     * @param inputSource the inputSource.
     * @return Document.
     * @throws DOMParserException if an SAXException or IOException occurs.
     */
    public Document parse(final InputSource inputSource) throws DOMParserException {
        return parse(inputSource, null, getDefaultEntityResolver());
    }
}
