/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 23.04.2008
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.xml;

/**
 * Class DOMSerializerOptions encapsulates settings for the DOMSerializer.
 *
 * @author Marko Reiprecht
 *         Created on 23.04.2008
 * @version $Revision: $
 */
public class DOMSerializerOptions {

    // ------------------------------ FIELDS ------------------------------

    private boolean m_omitXMLDeclaration;
    private String m_encoding;
    private boolean m_indent;
    private int m_indentAmount = 4;

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Method gets the defaultOptions.
     *
     * @return DOMSerializerOptions.
     */
    public static DOMSerializerOptions getDefaultOptions() {
        return new DOMSerializerOptions()
                .emitXMLDeclaration()
                .encodingUTF8()
                .noIndent();
    }

    /**
     * Method sets no indent.
     *
     * @return DOMSerializerOptions.
     */
    public DOMSerializerOptions noIndent() {
        m_indent = false;
        return this;
    }

    /**
     * Method gets the encoding.
     *
     * @return String.
     */
    public String getEncoding() {
        return m_encoding;
    }

    /**
     * Method gets the indent Amount.
     *
     * @return int.
     */
    public int getIndentAmount() {
        return m_indentAmount;
    }

    /**
     * Method gets if indent is enabled.
     *
     * @return boolean.
     */
    public boolean isIndent() {
        return m_indent;
    }

    /**
     * Method gets if xml declaration shall be written.
     *
     * @return boolean.
     */
    public boolean isOmitXMLDeclaration() {
        return m_omitXMLDeclaration;
    }

    // ------------------------ CANONICAL METHODS ------------------------

    /**
     * Method equals.
     *
     * @param o the o.
     * @return boolean.
     */
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof DOMSerializerOptions)) {
            return false;
        }

        final DOMSerializerOptions that = (DOMSerializerOptions) o;

        if (m_indent != that.m_indent) {
            return false;
        }
        if (m_indentAmount != that.m_indentAmount) {
            return false;
        }
        if (m_omitXMLDeclaration != that.m_omitXMLDeclaration) {
            return false;
        }
        if (m_encoding != null ? !m_encoding.equals(that.m_encoding) : that.m_encoding != null) {
            return false;
        }

        return true;
    }

    /**
     * Method hashCode.
     *
     * @return int.
     */
    public int hashCode() {
        int result;
        result = (m_omitXMLDeclaration ? 1 : 0);
        result = 31 * result + (m_encoding != null ? m_encoding.hashCode() : 0);
        result = 31 * result + (m_indent ? 1 : 0);
        result = 31 * result + m_indentAmount;
        return result;
    }

    /**
     * Method toString.
     *
     * @return String.
     */
    public String toString() {
        final StringBuffer sb = new StringBuffer();
        sb.append("org.n52.security.common.xml.DOMSerializerOptions");
        sb.append("{encoding='").append(getEncoding()).append('\'');
        sb.append(", isIndent=").append(isIndent());
        sb.append(", isOmitXMLDeclaration=").append(isOmitXMLDeclaration());
        sb.append(", indentAmount=").append(getIndentAmount());
        sb.append('}');
        return sb.toString();
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Method sets the XMLDeclaration shall be emited.
     *
     * @return DOMSerializerOptions.
     */
    public DOMSerializerOptions emitXMLDeclaration() {
        m_omitXMLDeclaration = false;
        return this;
    }

    /**
     * Method sets encoding.
     *
     * @param encoding the encoding.
     * @return DOMSerializerOptions.
     */
    public DOMSerializerOptions encoding(final String encoding) {
        m_encoding = encoding;
        return this;
    }

    /**
     * Method sets encoding to UTF-8.
     *
     * @return DOMSerializerOptions.
     */
    public DOMSerializerOptions encodingUTF8() {
        m_encoding = "UTF-8";
        return this;
    }

    /**
     * Method sets the indent amount.
     *
     * @param amount the amount.
     * @return DOMSerializerOptions.
     */
    public DOMSerializerOptions indentAmount(final int amount) {
        m_indentAmount = amount;
        if (m_indentAmount <= 0) {
            return noIndent();
        } else {
            return indent();
        }
    }

    /**
     * Method sets the indent is requested.
     *
     * @return DOMSerializerOptions.
     */
    public DOMSerializerOptions indent() {
        m_indent = true;
        return this;
    }

    /**
     * Method sets that XMLDeclaration shall be omited.
     *
     * @return DOMSerializerOptions.
     */
    public DOMSerializerOptions omitXMLDeclaration() {
        m_omitXMLDeclaration = true;
        return this;
    }
}
