/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 18.04.2008
 ******************************************************************************/
package org.n52.security.common.xml;

import javax.xml.namespace.QName;

/**
 * Interface XMLChunkHandler processes xml chunks.
 * <p/>
 * Xml chunks are valid xml documents which were part of an greater xml document.
 * <p/>
 * The interface is directly related to a XMLChunkReader, which reads an big xml document
 * and calls an XMLChunkHandler to process some of it's parts. A XMLChunkHandler can parse the
 * chunk with an DOMParser or an other XMLprocessing tool.
 * <p/>
 * If an XMLChunkHandler is responsible for a special xml element and within it's scope
 * other XMLChunkHandlers are registered, then the handler is informed about the processing of the
 * childs through the *Child* methods.
 *
 * @author Marko Reiprecht
 *         Created on 18.04.2008
 * @version $Revision: $
 */
public interface XMLChunkHandler {

    /**
     * Method handleChunkStart, informs about the start element of a chunk.
     *
     * @param handlerQName the handlerQName.
     */
    void handleChunkStart(final QName handlerQName);

    /**
     * Method is invoked if the end tag of the xml chunk is reached.
     *
     * @param handlerQName the handlerQName.
     * @param xmlChunk     the partialXml.
     */
    void handleChunkEnd(final QName handlerQName, final String xmlChunk);

    /**
     * Method is invoked if the start tag of an element, for which a child handler is registered
     * is reached.
     * The xmlChunk is not well formed, because the end tag is not closed.
     *
     * @param handlerQName the handlerQName.
     * @param xmlChunk     the xmlChunk.
     * @param childQName   the childQName.
     * @param childHandler the childHandler.
     */
    void handleChunkBeforChildHandler(final QName handlerQName, final String xmlChunk,
                                      final QName childQName, final XMLChunkHandler childHandler);

    /**
     * Method is invoked if the end tag of an element, for which a child handler is registered is reached.
     * <p/>
     * The xmlChunk is not well formed, because the end tag is not closed.
     *
     * @param handlerQName the handlerQName.
     * @param xmlChunk     the xmlChunk.
     * @param childQName   the childQName.
     * @param childHandler the childHandler.
     */
    void handleChunkAfterChildHandler(final QName handlerQName, final String xmlChunk,
                                      final QName childQName, final XMLChunkHandler childHandler);
}
